package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.license.ImppLicenseTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * @Description : 后台服务
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-06-19 下午2:32
 * @Modify:
 **/
@RestController
@Api(tags = "后台运维服务")
@RequestMapping("/impp/white/backstage")
public class BackstageController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(BackstageController.class);

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ICoreMemTreeService memTreeService;


    @GetMapping(value = "/redis-put-all")
    @ApiOperation(value = "重新加载所有缓存数据",notes = "重新加载所有缓存数据")
    public ResultBean redisPutAll(){
        try {
            systemInitService.putAndLoadAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-language")
    @ApiOperation(value = "重新加载语言数据",notes = "重新加载语言数据")
    public ResultBean redisPutSystemLanguage(){
        try {
            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-config")
    @ApiOperation(value = "重新加载系统配置数据",notes = "重新加载系统配置数据")
    public ResultBean redisPutSystemConfig(){
        try {
            systemInitService.putAndLoadSysConfig();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-dictionary")
    @ApiOperation(value = "重新加载系统字典数据",notes = "重新加载系统字典数据")
    public ResultBean redisPutSystemDictionary(){
        try {
            systemInitService.putAndLoadSysDictionary();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/reload-license")
    @ApiOperation(value = "重新加载授权数据",notes = "重新加载授权数据")
    public ResultBean reloadLicense(){
        try {
            ImppLicenseTool.cloneLicense();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(ImppLicenseTool.getLicense());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/reset/menu/soft-type")
    @ApiOperation(value = "重置菜单产品类型属性",notes = "重新加载授权数据")
    public ResultBean restSysMenuSoftType(){
        try {
            List<SysMenu> menuList = menuService.findAll();
            if(CollectionUtils.isNotEmpty(menuList)){

                /** 设置功能权限 softType 默认值*/
                Map<Long,SysMenu> menuMap = new HashMap<>();
                for (SysMenu menu : menuList) {
                    menu.setSoftType(CommonEnumUtil.SOFT_TYPE.IMPP.getValue());
                    menuMap.put(menu.getId(),menu);
                }

                List<SysMenu> treeMenus = memTreeService.packTreeSysMenu(menuList, CommonEnumUtil.SOFT_TYPE.IMPP.getMenuRootId());
                if(CollectionUtils.isNotEmpty(treeMenus)){
                    for (SysMenu menu : treeMenus) {
                        CommonEnumUtil.SOFT_TYPE softType = CommonEnumUtil.SOFT_TYPE.valueOfMenuRootId(menu.getId());

                        if(Objects.nonNull(softType)){
                            menu.setSoftType(softType.getValue());

                            List<Long> childMenus = memTreeService.findChildSysMenu(menu);
                            if(CollectionUtils.isNotEmpty(childMenus)){
                                for (Long key : childMenus) {
                                    if(menuMap.containsKey(key)){
                                        menuMap.get(key).setSoftType(softType.getValue());
                                    }
                                }
                            }
                        }
                    }
                }
                menuService.saveBatch(new ArrayList<>(menuMap.values()));
            }

            menuService.refreshRefSysMenuBySoftType();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
