package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionPlanService;
import cn.estsh.i3plus.core.api.iservice.busi.ISystemResourceService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.SysConfigRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysDictionaryRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ImppSystemTool;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.RESOURCE_PREFIX_MENU;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2019-01-11 12:34
 * @Modify:
 **/
@Service
public class SystemInitService implements ISystemInitService {

    private static final Logger LOGGER = LoggerFactory.getLogger(SystemLoginService.class);

    @Autowired
    private SysConfigRepository configRDao;

    @Autowired
    private SysDictionaryRepository dictionaryRDao;

    @Autowired
    public ISystemResourceService systemResourceService;

    @Autowired
    private ISysLocaleResourceService localeResourceService;

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ISysPojoVersionPlanService pojoVersionPlanService;


    @Resource(name= CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    private int type = ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue();

    @Override
    @ApiOperation(value = "初始化加载所有数据",notes = "初始化加载所有数据")
    public void putAndLoadAll() {
        try {

        type = ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue();

        LOGGER.info("加载 SysConfig 信息");
        putAndLoadSysConfig();
        LOGGER.info("加载 SysConfig 信息完成");

        LOGGER.info("加载 SysDictionary 信息");
        putAndLoadSysDictionary();
        LOGGER.info("加载 SysDictionary 信息完成");

        LOGGER.info("加载 SysLocaleLanguage 信息");
        putAndLoadSysLocaleLanguage();
        LOGGER.info("加载 SysLocaleLanguage 信息完成");

        LOGGER.info("加载ElasticSearch Client Url 信息");
        reloadElasticSearchClient();
        LOGGER.info("加载ElasticSearch Client Url 信息完成");

        LOGGER.info("加载 对象版本控制信息");
        reloadPojoVersionPlan();
        LOGGER.info("加载 对象版本控制信息 信息完成");
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    @ApiOperation(value = "加载系统配置",notes = "加载系统配置")
    public void putAndLoadSysConfig() {
        List<SysConfig> list = configRDao.findAll();
        if(list != null && list.size() > 0){
            list.forEach(config -> {
                RedisCacheTool.setSysConfigByConfigCode(config);
            });
            LOGGER.info("加载系统配置数量：【{}】",list.size());
        }
    }

    @Override
    @ApiOperation(value = "加载字典数据",notes = "加载字典数据")
    public void putAndLoadSysDictionary() {
        try {
            List<SysDictionary> list = dictionaryRDao.findAll();
            if(list != null && list.size() > 0){
                list.stream().filter(sysDictionary-> sysDictionary.getParentCodeRdd() != null).forEach(System.out :: print);
                Map<String, List<SysDictionary>> parentCodeMap = list.stream().filter(sysDictionary-> StringUtils.isNotBlank(sysDictionary.getParentCodeRdd())).collect(Collectors.groupingBy(SysDictionary::getParentCodeRdd));
                Map<Long, List<SysDictionary>> parentIdMap = list.stream().collect(Collectors.groupingBy(SysDictionary::getParentId));

                for (String key : parentCodeMap.keySet()) {
                    //存放于缓存
                    RedisCacheTool.setSysDictionaryListByCode(key, parentCodeMap.get(key));
                }

                for (long key : parentIdMap.keySet()) {
                    RedisCacheTool.setSysDictionaryListByParentId(key,parentIdMap.get(key));
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    @ApiOperation(value = "加载语言数据",notes = "加载语言数据")
    public void putAndLoadSysLocaleLanguage() {
        // 加载资源
        putAndLoadResource();

        // 加载WEB资源
        putAndLoadWebResource();

        //加载语言数据
        putAndLoadSysMenu();

        // 加载内存数据
        putAndLoadRAMResource();
    }

    public void putAndLoadWebResource() {
        // Redis 缓存
        if (type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()) {
            /**
             * 整体数据结构：
             * code <langCode : value>
             *    先根据code查询，判断是否存在，然后通过语言获取对应的值
             *
             * 语言代码 hashMap结构
             * langCode < code : value>
             *
             * 语言代码+模块 hashMap结构
             * langCode + softType < code : value>
             *
             */
            // 查询通用与前端资源数据
            SysLocaleResource localeResource = new SysLocaleResource();
            localeResource.setResourceTypes(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.COMMON.getValue() + "," + CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.WEB.getValue());

            Integer resCount = systemResourceService.getSysLocaleResourceCount(localeResource);

            //语言数量
            List<SysLocaleLanguage> langList = systemResourceService.listSysLocaleLanguage(null);

            LOGGER.info("【加载WEB资源】共有{}个资源，{}种语言。", resCount, langList.size());
            Pager pager = new Pager(resCount, 5000);
            pager.setCurrentPage(0);
            //查询所有资源
            ListPager<SysLocaleResource> resListPage;
            // 前端+语言+模块
            Map<String, Map<String, String>> webLangSoftMap = new HashMap<>();
            // 前端+语言
            Map<String, Map<String, String>> webLangMap = new HashMap<>();
            // resKey
            String resKey;
            // key + value
            Map<String, String> resMap = new HashMap<>();

            do {
                pager.next();
                resListPage = systemResourceService.listSysLocaleResourceByPager(localeResource, pager);

                for (SysLocaleResource res : resListPage.getObjectList()) {
                    resKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + res.getLanguageCode();
                    resMap = webLangMap.computeIfAbsent(resKey, k -> new HashMap<>());
                    resMap.put(res.getResourceKey(), res.getResourceValue());

                    resKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + res.getLanguageCode() + ":" + res.getSoftType();
                    resMap = webLangSoftMap.computeIfAbsent(resKey, k -> new HashMap<>());
                    resMap.put(res.getResourceKey(), res.getResourceValue());
                }


                LOGGER.info("【加载WEB资源】数据分类中 {} / {} ", pager.getEndRow(), pager.getTotalRows());
            } while (pager.hasNext());

            for (Map.Entry<String, Map<String, String>> entry : webLangMap.entrySet()) {
                redisRes.putHashMap(entry.getKey(), entry.getValue(),-1);
            }
            LOGGER.info("【加载WEB资源】WEB-语言 已分类缓存完毕");

            for (Map.Entry<String, Map<String, String>> entry : webLangSoftMap.entrySet()) {
                redisRes.putHashMap(entry.getKey(), entry.getValue(),-1);
            }
            LOGGER.info("【加载WEB资源】WEB-语言-模块 已分类缓存完毕");

            LOGGER.info("【加载WEB资源】加载完成。");
        }
    }

    public void putAndLoadResource() {
        // Redis 缓存
        if (type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()) {
            /**
             * 整体数据结构：
             * code <langCode : value>
             *    先根据code查询，判断是否存在，然后通过语言获取对应的值
             *
             * 语言代码 hashMap结构
             * langCode < code : value>
             *
             * 语言代码+模块 hashMap结构
             * langCode + softType < code : value>
             *
             */
            Integer resCount = systemResourceService.getSysLocaleResourceCount(null);

            //语言数量
            List<SysLocaleLanguage> langList = systemResourceService.listSysLocaleLanguage(null);

            LOGGER.info("【加载资源】共有{}个资源，{}种语言。", resCount, langList.size());
            Pager pager = new Pager(resCount, 5000);
            pager.setCurrentPage(0);
            //查询所有资源
            ListPager<SysLocaleResource> resListPage;
            // 语言+模块
            Map<String, Map<String, String>> langSoftMap = new HashMap<>();
            // resKey
            String resKey;
            // key + value
            Map<String, String> resMap = new HashMap<>();

            do {
                pager.next();
                resListPage = systemResourceService.listSysLocaleResourceByPager(null, pager);

                for (SysLocaleResource res : resListPage.getObjectList()) {
                    resKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + res.getSoftType() + ":" + res.getResourceKey();
                    resMap = langSoftMap.computeIfAbsent(resKey, k -> new HashMap<>());
                    resMap.put(res.getLanguageCode(), res.getResourceValue());
                }

                pager.resetEndRow();
                LOGGER.info("【加载资源】数据分类中 {} / {} ", pager.getEndRow(), pager.getTotalRows());
            } while (pager.hasNext());

            for (Map.Entry<String, Map<String, String>> entry : langSoftMap.entrySet()) {
                redisRes.putHashMap(entry.getKey(), entry.getValue(),-1);
            }
            LOGGER.info("【加载资源】模块资源 已分类缓存完毕");

            LOGGER.info("【加载资源】加载完成。");
        }
    }

    public void putAndLoadSysMenu() {

        SysLocaleResource resource = new SysLocaleResource();
        resource.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
        List<SysLocaleResource> resources = localeResourceService.findSysLocaleResource(resource);
        List<SysMenu> menus = menuService.findAll();
        List<SysLocaleLanguage> languages = sysLocaleLanguageRDao.findAll();

        Map<String,SysLocaleResource> resourceKeyMap = new HashMap<>();
        List<SysLocaleResource> insertList = new ArrayList<>();
        Map<String,String> redisCacheMap = new HashMap<>();

        if(resources != null && resources.size() > 0){
            resources.forEach(res -> resourceKeyMap.put(StringUtils.join(res.getLanguageCode(),res.getResourceKey()),res));
        }
        String redisLanguageKey;
        SysLocaleResource res;
        if(menus != null && menus.size() > 0 && languages != null && languages.size() > 0){
            for (SysLocaleLanguage language : languages) {
                 redisLanguageKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_MENU + ":" + language.getLanguageCode();

                for (SysMenu menu : menus) {
                    // 多次校验
                    res = resourceKeyMap.get(StringUtils.join(language.getLanguageCode(),RESOURCE_PREFIX_MENU,menu.getName()));
                    if(res == null){
                        res = localeResourceService.getSysLocaleResourceByLanguageCodeAndResKey(language.getLanguageCode(),
                                RESOURCE_PREFIX_MENU + menu.getName());
                    }

                    if(res == null){
                        // 当系统没有相应的功能权限资源时创建资源
                        res = new SysLocaleResource();
                        res.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
                        res.setLanguageCode(language.getLanguageCode());
                        res.setLanguageNameRdd(language.getLanguageName());
                        res.setResourceKey(RESOURCE_PREFIX_MENU + menu.getName());
                        res.setResourceValue(menu.getName());
                        res.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                        ConvertBean.serviceModelInitialize(res, PlatformConstWords.SYSTEM_INIT_USER);

                        insertList.add(res);
                    }

                    resourceKeyMap.put(StringUtils.join(res.getLanguageCode(),res.getResourceKey()),res);
                    redisCacheMap.put(res.getResourceKey(),res.getResourceValue());
                }

                redisRes.putHashMap(redisLanguageKey,redisCacheMap,0);
                redisCacheMap.clear();
            }
        }

        if(insertList != null && insertList.size() > 0){
            localeResourceService.saveSysLocaleResource(insertList);
        }
    }

    public void putAndLoadRAMResource(){
        // 清除内存中的资源缓存
        menuService.refreshMemSysMenuLanguage();
        LocaleUtils.cleanCacheResMap();
    }

    /**
     * 缓存存放数据统一管理
     * @param key
     * @param data
     * @param dataType
     */
    private void putDataToCache(String key,Object data,Class dataType){
        if(type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()){
            LOGGER.info(" Put Cache Redis Key:{},value:{},dataType:{}",key,data,dataType);
            // Redis 缓存
            if(dataType == List.class){
                redisRes.putObject(key,data,-1);
            }else if(dataType == Object.class){
                redisRes.putObject(key,data,-1);
            }else{
                LOGGER.error("不支持树形{}存放缓存！",dataType);
            }
        }
    }

    /**
     * 从缓存获取对象数据
     * @param key
     * @param dataType
     * @return
     */
    @Override
    @ApiOperation(value = "获取缓存中的数据",notes = "获取缓存中的数据")
    public Object getDataFromCache(String key,Class dataType){
        if(type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()){
            // Redis 缓存
            if(dataType == List.class){
                return redisRes.getList(key,0,-1);
            }else if(dataType == SysConfig.class){
                return redisRes.getObject(key);
            }else{
                LOGGER.error("不支持树形{}存放缓存！",dataType);
            }
        }
        return null;
    }

    @Override
    public void reloadElasticSearchClient() {
        String url = RedisCacheTool.getSysConfigStrVal(CommonConstWords.PROP_IMPP_ELASTICSEARCH_CONFIG_URL);
        if(StringUtils.isBlank(url)){
            if(Objects.nonNull(ImppSystemTool.getEnvironment())){
                url = ImppSystemTool.getEnvironment().getProperty(CommonConstWords.PROP_IMPP_ELASTICSEARCH_URL);
            }
        }

        if(StringUtils.isBlank(url)){
            url = CommonConstWords.PROP_IMPP_ELASTICSEARCH_DEFAULT_URL;
        }

        LOGGER.info("加载ElasticSearch Client Url:{}",url);
        ElasticSearchTool.init(url);
    }

    @Override
    public void reloadPojoVersionPlan() {
        List<SysPojoVersionPlan> list = pojoVersionPlanService.findAll();
        if(CollectionUtils.isNotEmpty(list)){
            for (SysPojoVersionPlan plan : list) {
                String redisKey = CommonConstWords.REDIS_PREFIX_POJO_VERSION + ":" + plan.getSoftType() + ":" + plan.getRefClass();
                redisRes.putObject(redisKey, JSON.toJSONString(plan),-1);
            }
        }
    }
}
