package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDataSourceService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SysDataSource;
import cn.estsh.i3plus.pojo.platform.repository.SysDataSourceRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.CacheCrudService;
import cn.estsh.impp.framework.boot.datasource.ImppJdbcTemplateFactory;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.alibaba.fastjson.JSON;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-5-11 上午11:12
 * @Modify:
 **/
@Service
public class SysDataSourceService extends CacheCrudService<SysDataSource> implements ISysDataSourceService {

    @Autowired
    private SysDataSourceRepository dataSourceRDao;

    @Override
    public BaseRepository<SysDataSource, Long> getRepository() {
        return dataSourceRDao;
    }

    @Override
    public void checkSysDataSourceOnly(SysDataSource bean) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean);
        DdlPreparedPack.getNumNOEqualPack(bean.getId(),"id",packBean);
        DdlPreparedPack.getNumEqualPack(bean.getSoftType(),"softType",packBean);
        DdlPreparedPack.getStringEqualPack(bean.getSourceCode(),"sourceCode",packBean);

        if(dataSourceRDao.findByHqlWhereCount(packBean) > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("数据代码已存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void packAfterRefreshCache() {
        List<SysDataSource> list = getMemCacheDataSource();
        ImppRedis imppRedis = getRedisBusiCache();
        if(CollectionUtils.isNotEmpty(list)){
            if(Objects.nonNull(imppRedis)){
                for (SysDataSource source : list) {
                    String redisKey = ImppJdbcTemplateFactory.getDataSourceName(source.getSourceName());
                    imppRedis.putObject(redisKey, JSON.toJSONString(source), -1);
                }
            }else{
                LOGGER.info("同步数据到错误:无法获取Redis连接信息");
            }
        }
    }

    @Override
    public DdlPackBean getFindPagerPackBean(SysDataSource bean) {
        return CoreHqlPack.packSysDataSource(bean);
    }
}
